/**
 * Represents a node in a doubly-linked list,
 * which can store data of type E.
 */
class ListNode<E> {
  private E data;
  public ListNode<E> next;
  public ListNode<E> prev;

  /**
   * Constructs a linked list node, saving the data.
   * The next and prev pointers are initially null.
   * 
   * @param data - data to store at this node.
   */
  public ListNode(E data) {
    this.data = data;
    prev = null;
    next = null;
  }

  /**
   * @return the data stored at this node.
   */
  public E get() {
    return data;
  }
}

/**
 * Represents a doubly-linked list,
 * where each node can store data of type E.
 */
public class LinkedList<E> {
  private ListNode<E> head;

  /**
   * Creates an empty linked list.
   */
  public LinkedList() {
    head = null;
  }

  /**
   * Adds a node to the beginning of the list.
   * 
   * @param data - data to store at this node.
   */
  public void addFirst(E data) {
    ListNode<E> node = new ListNode<E>(data);
    node.next = head;
    head = node;
  }

  /**
   * Removes the node at the beginning of the list.
   */
  public void removeFirst() {
    if (head != null) {
      head = head.next;
    }
  }

  // public E get(int index)

  // public int size()

  // public void addLast(E data)

  // public void removeLast()

  // public void add(E data)

  // public void add(int index, E data)

  // public void remove(int index)

  /**
   * @return a String representation of the linked list.
   */
  public String toString() {
    String result = "";
    ListNode<E> node = head;
    while (node != null) {
      result += node.get().toString();
      if (node.next != null)
        result += " -> ";
      node = node.next;
    }
    return result;
  }

  /**
   * Tests the linked list implementation.
   * 
   * @param args
   */
  public static void main(String[] args) {
    LinkedList<Integer> list = new LinkedList<>();

    list.addFirst(5);
    list.addFirst(3);
    list.addFirst(2);
    list.removeFirst();
    System.out.println(list);
  }
}